@@ -18,3 +18,7 @@ using namespace std;
 #include "icenReg_files/basicUtilities.cpp"
 #include "icenReg_files/ic_par.cpp"
 #include "icenReg_files/ic_sp_ch.cpp"
+#include "icenReg_files/bivariateNPMLE.cpp"
+#include "icenReg_files/ic_sp_gradDescent.cpp"
+
+#include "icenReg_files/experimentalCode.cpp"
