@@ -32,15 +32,15 @@ public:
     double pob;
 };
 
-class actPointInf{
+/*class actPointInf{
 public:
     int ind;                   //beginning and ending of parameters affected by adjusting active points
     double par;
     vector<int> dep_obs;
 //    vector<int> dep_nodes;      //don't think this is necessary; l, r tells us all this!
-};
+};  */
 
-class actSet_Abst{
+class icm_Abst{
 public:
     void update_p_ob(int i);    //done, not checked
     
@@ -48,72 +48,29 @@ public:
     // calculates the entire likelihood function.
     // Does not update eta or hazards!
     
-    double par_llk(int a_ind);     //done, not checked
+    double par_llk(int ind);     //done, not checked
     // only calculates partial likelihood based on an active index
     
-    void act_setPar(int act_i, double val);     //done, not checked
-    //updates the active index act_i according to an active set update
-    //updates baseline hazards and dependent observations
-    
-    void act_addPar(int act_i, double delta){
-        double newVal = actIndex[act_i].par + delta;
-        act_setPar(act_i, newVal);
-    }
-    
-    void act_addPar(vector<double> &delta);
-    
-    void addDepNodes(vector<int> &intoVec, int l, int r);
-    // adds dependent obs into vector for use in par_llk
-    
     vector<obInf> obs_inf;
     vector<node_info> node_inf;
     
-    vector<actPointInf> actIndex;
-    //vector of active point infos
-    int getNextRawActInd(int act_i);
+    void numericBaseDervsAllRaw(vector<double> &d1, vector<double> &d2);
     
-    void update_etas();
-
-    
-    void uniActiveOptim(int raw_ind);
-    //univariate update of active points
+    void icm_addPar(vector<double> &delta);
 
     void numericBaseDervsOne(int raw_ind, vector<double> &d);
     void numericBaseDervsAllAct(vector<double> &d1, vector<double> &d2);
-    void numericBaseDervsAllRaw(vector<double> &d1, vector<double> &d2);
 
-    void analyticBase1stDerv(vector<double> &d1);
     
-    void vem_step();
+    void update_etas();
+    
     void icm_step();
     
-    
     void numericRegDervs();
     void covar_nr_step();
     
-    int getActInd(int raw_ind);
-    //takes in a raw index and gives back the corresponding active index.
-    //returns -1 if not an index
-    
-    int getRawInd(int act_ind){return(actIndex[act_ind].ind);}
-    //returns the raw index from the i_th active index
-    
-    
-    
-    void addActive(int raw_ind, double par);        //done
-    //add active point
-
-    void addActive(int raw_ind){ addActive(raw_ind, baseCH[raw_ind]);}
-    // adds active point without adjusting it
-    
-    void removeActive(int act_ind); //done
-    //remove active point
-    
-    int getNextActRawInd(int raw_ind);
-    
-    void checkIfActShouldDrop(int act_ind); //checks an active point and sees if it should be dropped
-    
     virtual double basHaz2CondS(double ch, double eta) = 0;     //done
+    virtual double baseS2CondS(double s, double eta) = 0;
     virtual double base_d1_contr(double h, double pob, double eta) = 0; //done, not checked
     virtual double reg_d1_lnk(double ch, double xb, double log_p) = 0;
     virtual double reg_d2_lnk(double ch, double xb, double log_p) = 0;
@@ -123,8 +80,10 @@ public:
     
     Eigen::VectorXd     baseCH;     //Vector of baseline log cumulative hazards.
                                     //baseH[0] fixed to -Inf, baseH[k-1] = Inf
-    Eigen::VectorXd     H_d1;       //Vector of derivatives for CH's
-    Eigen::MatrixXd     H_d2;       //Hessian for CH's
+    Eigen::VectorXd     backupCH;   //used to save values in optimization steps
+    Eigen::VectorXd     propVec;    //used for proposition step during NR update on regression parameters
+ /*   Eigen::VectorXd     H_d1;       //Vector of derivatives for CH's
+    Eigen::MatrixXd     H_d2;       //Hessian for CH's          */
     Eigen::VectorXd     base_p_obs; //Baseline probability of each observation  //initialized
     Eigen::VectorXd     etas;       //linear combination of regression parameters   //initialized
     Eigen::VectorXd     expEtas;    //exp(etas) //initialized
@@ -132,27 +91,82 @@ public:
     Eigen::MatrixXd     covars;     //covariates        //initialized
     Eigen::VectorXd     reg_d1;     //first derivatives of regression parameters        //initialized
     Eigen::MatrixXd     reg_d2;     //Hessian for derivatives       //initialized
-
+//    Eigen::VectorXd     reg_d2;     //second derivatives: ignoring off diagonals!
+    
     vector<double> w;
     
+    double maxBaseChg;      //Max change in baseline parameters during icm step
     double h;
     bool hasCovars;
+    
+    bool startGD;
+    vector<double> baseS;
+    vector<double> baseP;
+    vector<double> d_cond_S_left;
+    vector<double> d_cond_S_right;
+    vector<double> base_p_derv;
+    vector<double> prop_p;
+    double llk_from_p();
+    double numeric_p_der(int i);
+    
+    double dervConS_fromBaseS(double s, double eta);
+    void baseCH_2_baseS();
+    void baseS_2_baseP();
+    void baseP_2_baseS();
+    void baseS_2_baseCH();
+    void calc_cond_S_derv();
+    void calc_base_p_derv();
+    double getMaxScaleSize( vector<double> &p, vector<double> &prop_p);
+    void gradientDescent_step();
+    
+    void numeric_dobs_dp();
+    double cal_log_obs(double s1, double s2, double eta);
+    
+    vector<bool> usedVec;
+    
+    double almost_inf;
+    int failedGA_counts;
+    int iter;
+    int numBaselineIts;
+    bool useFullHess;
+    
+    double exchangeAndUpdate(double delta, int i1, int i2);
+    // REQUIRES baseP BEING UP TO DATE!!!
+    
+    vector<int> exchangeIndices;
+    
+    void last_p_update();
+    void vem();
+    void exchange_p_opt(int i1, int i2);
+    void vem_sweep();
+    void vem_sweep2();
 };
 
-void setupActSet(SEXP Rlind, SEXP Rrind, SEXP RCovars, SEXP R_w, actSet_Abst* actSet);
+void setup_icm(SEXP Rlind, SEXP Rrind, SEXP RCovars, SEXP R_w, icm_Abst* icm_obj);
 //function for setting up a actSet_Abst class
 
-void addDepNodes(vector<int> &intoVec, int l, int r, vector<node_info> &nf);
+//void addDepNodes(vector<int> &intoVec, int l, int r, vector<node_info> &nf);
 //
 
 void cumhaz2p_hat(Eigen::VectorXd &ch, vector<double> &p);
 
-class actSet_ph : public actSet_Abst{
+class icm_ph : public icm_Abst{
 public:
     double basHaz2CondS(double ch, double eta){
         if(ch == R_NegInf)  return(1);
         if(ch == R_PosInf)  return(0);
         return(exp(-exp(ch + eta) )) ;}
+    
+    double baseS2CondS(double s, double eta){
+        if(s == 1.0) return(1.0);
+        if(s == 0.0) return(0.0);
+/*        double expEta = exp(eta);
+        double ans = pow(s, expEta);    */
+        double logCH = log( -log(s) );
+        double ans = exp(-exp(logCH + eta));
+        return(ans);
+    }
+    
     double base_d1_contr(double ch, double pob, double eta){
         double expVal = -exp(eta + ch);
         double logAns = eta + ch + expVal - log(pob);
@@ -168,11 +182,11 @@ public:
         double term2 = exp(term1 - log_p);
         return(term1 * term2 + term1 * term1 *term2);
     }
-    virtual ~actSet_ph(){};
+    virtual ~icm_ph(){};
 };
 
 
-class actSet_po : public actSet_Abst{
+class icm_po : public icm_Abst{
 public:
     double basHaz2CondS(double ch, double eta){
         if(ch == R_NegInf)  return(1);
@@ -181,6 +195,13 @@ public:
         double s = exp(-mu);
         double s_nu = exp(eta - mu);
         return( (s_nu) / (s_nu - s + 1)) ;}
+    
+    double baseS2CondS(double s, double eta){
+        double nu = exp(eta);
+        double s_nu = s * nu;
+        return((s_nu)/ (s_nu - s + 1) );
+    }
+    
     double base_d1_contr(double ch, double pob, double eta){
         double s = exp(-exp(ch));
         double s_nu = exp(eta - exp(ch));
@@ -204,11 +225,11 @@ public:
         double ans = top/(bottom * exp(log_p));
         return(ans);
     }
-    virtual ~actSet_po(){};
+    virtual ~icm_po(){};
 };
 
 extern "C" {
-    SEXP ic_sp_ch(SEXP Rlind, SEXP Rrind, SEXP Rcovars, SEXP fitType, SEXP R_w);
+    SEXP ic_sp_ch(SEXP Rlind, SEXP Rrind, SEXP Rcovars, SEXP fitType, SEXP R_w, SEXP R_use_GD, SEXP R_maxiter, SEXP R_baselineUpdates, SEXP R_useFullHess, SEXP R_useExpSteps);
     SEXP findMI(SEXP R_AllVals, SEXP isL, SEXP isR, SEXP lVals, SEXP rVals);
 }
 #endif /* defined(____ic_sp_cm__) */
